

/********************************************/
/* Matrix.h - by Morgan Aldridge			*/
/********************************************/
/*   This source code is copyright (C) 2001 */
/* by Morgan Aldridge.  You are free to use */
/* and/or distibute this source code as long*/
/* as it remains in it's original form and  */
/* this message is included					*/
/********************************************/
/* If you find a bug in this code, then     */
/* e-mail classic@sover.net so that Morgan  */
/* Aldridge may fix any bugs that may remain*/
/********************************************/
/* This source code is not guaranteed to    */
/* work perfectly, so use it at your own    */
/* risk.  Morgan Aldridge will not be held  */
/* responsible to any damage which may      */
/* occur due to your use of this source code*/
/********************************************/

#ifndef _MATRIX_H_
#define	_MATRIX_H_

/**> HEADER FILES <**/
#include <math.h>
#include <stdio.h>
//#include "3DTypes.h"

/* Uncomment for OpenGL compliance */
#define	OPENGL_STYLE_MATRIX
/* Uncomment for normal matrices */
/* #define	NORMAL_STYLE_MATRIX */

/**> CONSTANT DECLARATIONS <**/
#ifdef	OPENGL_STYLE_MATRIX
#define	INDEX(r,c)				(((c-1)*4)+(r-1))	/* Macro for a matrix element's index */
#elif defined( NORMAL_STYLE_MATRIX )
#define	INDEX(r,c)				(((r-1)*4)+(c-1))	/* Macro for a matrix element's index */
#endif

typedef struct Vector3D
{
	double	x;
	double	y;
	double	z;
}	Vector3D;

typedef struct Matrix4
{
	double	m[16];
}	Matrix4;

/**> FUNCTION PROTOTYPES <**/
/* Functions for manipulating matrices */
void		Matrix_Identity( Matrix4 *matrix );
Matrix4		Matrix_Multiply( Matrix4 matrix1, Matrix4 matrix2 );
Matrix4		Matrix_Translate( Matrix4 matrix, double x, double y, double z );
Matrix4		Matrix_Rotate( Matrix4 matrix, double x, double y, double z );
Matrix4		Matrix_Scale( Matrix4 matrix, double sX, double sY, double sZ );
Vector3D	Matrix_MultiplyVertex( Matrix4 matrix, double x, double y, double z );

/* Functions that create various types of matrices */
Matrix4		Matrix_TranslationMatrix( double x, double y, double z );
Matrix4		Matrix_XRotationMatrix( double x );
Matrix4		Matrix_YRotationMatrix( double y );
Matrix4		Matrix_ZRotationMatrix( double z );
Matrix4		Matrix_ScalingMatrix( double sX, double sY, double sZ );

/* Debugging functions */
void		Matrix_Print( Matrix4 matrix );

#endif
